package net.wanji.com.controller;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import lombok.RequiredArgsConstructor;
import net.wanji.com.service.controller.StaticInfoService;
import net.wanji.common.annotation.aspect.AspectLog;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.framework.rest.JsonViewObject;
import net.wanji.databus.dao.entity.CrossSchedulesPO;
import net.wanji.databus.dto.CrossInfoDTO;
import net.wanji.databus.dto.CrossSchedulesDTO;
import net.wanji.databus.dto.PlanSectionDTO;
import net.wanji.databus.dto.SchemePhaseLightsDTO;
import net.wanji.databus.po.CrossInfoPO;
import net.wanji.databus.vo.PlanSectionVO;
import net.wanji.databus.vo.SchemePhaseLightsVO;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.core.MediaType;
import java.util.List;
import java.util.Objects;

/**
 * 静态信息接口
 *
 * @date 2022/11/15 9:38
 */
@Api(value = "静态信息接口", tags = "静态信息接口")
@RequestMapping("/staticInfo")
@RestController
@RequiredArgsConstructor
public class StaticInfoController {

    private final StaticInfoService staticInfoService;

    @AspectLog(description = "信号路口基础信息", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/crossInfo",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "信号路口基础信息", notes = "信号路口基础信息", response = CrossInfoPO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossInfoPO.class)
    })
    public JsonViewObject crossInfo(@RequestBody @Validated CrossInfoDTO crossInfoDTO) throws Exception {
        List<CrossInfoPO> crossInfoPOList = staticInfoService.crossBasicInfo(crossInfoDTO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(crossInfoPOList);
    }

    @AspectLog(description = "方案数据-方案信息、相位信息、灯组信息", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/schemePhaseLights",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "方案数据-方案信息、相位信息、灯组信息", notes = "方案数据-方案信息、相位信息、灯组信息",
            response = SchemePhaseLightsVO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SchemePhaseLightsVO.class)
    })
    public JsonViewObject schemePhaseLights(@RequestBody @Validated SchemePhaseLightsDTO schemePhaseLightsDTO) throws Exception {
        // 更新数据库
        staticInfoService.schemePhaseLights(schemePhaseLightsDTO);
        // 构造返回值
        SchemePhaseLightsVO schemePhaseLightsVO = staticInfoService.buildSchemePhaseLightsResponse(schemePhaseLightsDTO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(schemePhaseLightsVO);
    }

    @AspectLog(description = "计划数据-计划信息、时段信息", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/planSection",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "计划数据-计划信息、时段信息", notes = "计划数据-计划信息、时段信息", response = PlanSectionVO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = PlanSectionVO.class)
    })
    public JsonViewObject planSection(@RequestBody @Validated PlanSectionDTO planSectionDTO) throws Exception {
        // 更新数据库
        staticInfoService.planSection(planSectionDTO);
        // 构造返回值
        String crossId = planSectionDTO.getCrossId();
        Integer planNo = planSectionDTO.getPlanNo();
        PlanSectionVO planSectionVO = new PlanSectionVO();
        if (Objects.isNull(planNo) || planNo == -1) {
            // 返回所有
            planSectionVO = staticInfoService.buildPlanSectionResponse(crossId);
        } else {
            // 按计划号返回
            planSectionVO = staticInfoService.buildPlanSectionResponse(crossId, planNo);
        }
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(planSectionVO);
    }

    @AspectLog(description = "时间表数据", operationType = BaseEnum.OperationTypeEnum.QUERY)
    @PostMapping(value = "/crossSchedules",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "时间表数据", notes = "时间表数据", response = CrossSchedulesPO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossSchedulesPO.class)
    })
    public JsonViewObject crossSchedules(@RequestBody @Validated CrossSchedulesDTO crossSchedulesDTO) throws Exception {
        // 更新数据库
        staticInfoService.crossSchedules(crossSchedulesDTO);
        // 构造返回值
        List<String> crossIdList = crossSchedulesDTO.getCrossIdList();
        List<CrossSchedulesPO> crossSchedulesPOList = staticInfoService.buildCrossSchedulesResponse(crossIdList);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(crossSchedulesPOList);
    }
}
